/*
 Copyright (c) 2013 yvt
 
 This file is part of OpenSpades.
 
 OpenSpades is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 OpenSpades is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with OpenSpades.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
 namespace spades {
	class ViewSMGSkin: 
	IToolSkin, IViewToolSkin, IWeaponSkin,
	BasicViewWeapon {
		
		private AudioDevice@ audioDevice;
		private Model@ gunModel;
		private Model@ magazineModel;
		
		private AudioChunk@ fireSound;
		private AudioChunk@ reloadSound;
		
		ViewSMGSkin(Renderer@ r, AudioDevice@ dev){
			super(r);
			@audioDevice = dev;
			@gunModel = renderer.RegisterModel
				("Models/Weapons/SMG/WeaponNoMagazine.kv6");
			@magazineModel = renderer.RegisterModel
				("Models/Weapons/SMG/Magazine.kv6");
			
			@fireSound = dev.RegisterSound
				("Sounds/Weapons/SMG/Fire.wav");
			@reloadSound = dev.RegisterSound
				("Sounds/Weapons/SMG/Reload.wav");
		}
		
		void Update(float dt) {
			BasicViewWeapon::Update(dt);
		}
		
		void WeaponFired(){
			BasicViewWeapon::WeaponFired();
			
			if(!IsMuted){
				Vector3 origin = Vector3(0.4f, 9.f, 0.5f);
				AudioParam param;
				param.volume = 8.f;
				audioDevice.PlayLocal(fireSound, origin, param);
			}
		}
		
		void ReloadingWeapon() {
			if(!IsMuted){
				Vector3 origin = Vector3(0.4f, -0.3f, 0.8f);
				AudioParam param;
				param.volume = 0.2f;
				audioDevice.PlayLocal(reloadSound, origin, param);
			}
		}
		
		float GetZPos() {
			return 0.2f - AimDownSightStateSmooth * 0.01f;
		}
		
		void Draw2D() {
			if(AimDownSightStateSmooth > 0.8){
				Image@ img = renderer.RegisterImage("Gfx/smg.png");
				float height = renderer.ScreenHeight;
				float width = height * (800.f / 600.f); 
				renderer.Color = (Vector4(1.f, 1.f, 1.f, 1.f));
				renderer.DrawImage(img,
					AABB2((renderer.ScreenWidth - width) * 0.5f,
							(renderer.ScreenHeight - height) * 0.5f,
							width, height));
				return;
			}
			
			BasicViewWeapon::Draw2D();
		}
		
		void AddToScene() {
			if(AimDownSightStateSmooth > 0.8){
				LeftHandPosition = Vector3(6.f, 6.f, 7.f);
				RightHandPosition = Vector3(1.f, 3.f, 7.f);
				return;
			}
		
			Matrix4 mat = CreateScaleMatrix(0.025f);
			mat = GetViewWeaponMatrix() * mat;
			
			bool reloading = IsReloading;
			float reload = ReloadProgress;
			Vector3 leftHand, rightHand;
			
			leftHand = mat * Vector3(-1.f, 11.f, 4.5f);
			rightHand = mat * Vector3(-3.5f, -2.1f, 1.f);
			
			Vector3 position1 = mat * Vector3(-1.f, 11.f, 4.5f);
			Vector3 position2 = mat * Vector3(-1.f, 1.f, 4.5f);
			Vector3 position3 = mat * Vector3(-1.f, 1.f, 45.f);
			
			ModelRenderParam param;
			param.matrix = eyeMatrix * mat;
			param.depthHack = true;
			renderer.AddModel(gunModel, param);
			
			// magazine/reload action
			if(reloading) {
				if(reload < 0.1f){
					// move hand to magazine
					float per = reload / 0.1f;
					leftHand = Mix(position1, position2, SmoothStep(per));
				}else if(reload < 0.5f){
					// move hand
					float per = reload / 0.5f;
					leftHand = Mix(position2, position3, SmoothStep(per));
					// magazine release
					mat *= CreateTranslateMatrix(Mix (Vector3(0.f, 0.f, 0.f), Vector3(0.f, 0.f, 45.f), per));
				}else if(reload < 0.6f){
					// move hand
					float per = reload / 0.6f;
					leftHand = Mix(position3, position2, SmoothStep(per));
					// magazine insert
					mat *= CreateTranslateMatrix(Mix (Vector3(0.f, 0.f, 45.f), Vector3(0.f, 0.f, 0.f), per));
				}else if(reload < 0.8f){
					// move the hand to the original position
					float per = reload / 0.8f;
					leftHand = Mix(position2, position1, SmoothStep(per));
				}
			}
			
			param.matrix = eyeMatrix * mat;
			renderer.AddModel(magazineModel, param);
			
			LeftHandPosition = leftHand;
			RightHandPosition = rightHand;
		}
	}
	
	IWeaponSkin@ CreateViewSMGSkin(Renderer@ r, AudioDevice@ dev) {
		return ViewSMGSkin(r, dev);
	}
}
